import json

from notebuild.tool.fastapi import add_api_routes, api_route
from notecoin.database.base import session
from notecoin.database.connect import RedisConnect
from notecoin.okex.database.client import OkexClientAccountBalance
from notecoin.okex.server.const import market_tickers
from notecoin.strategy.domain import OkexCoin


class AutoSeller(RedisConnect):
    def __init__(self, prefix="/sell", *args, **kwargs):
        self.total = 0
        self.usdt = 0
        self.coin_map = {}
        super(AutoSeller, self).__init__(prefix=prefix, *args, **kwargs)
        add_api_routes(self)

    def load_account(self):
        self.coin_map = {}
        details = session.query(OkexClientAccountBalance).filter(OkexClientAccountBalance.eqUsd > 10).all()
        for _detail in details:
            detail = _detail.json()

            if detail['ccy'] == 'USDT':
                self.usdt = float(detail['availBal'])
                self.total = self.usdt
                continue
            coin = OkexCoin.instance_by_account(detail)
            if coin.money > 1:
                self.coin_map[coin.coin_id] = coin

    def update_price(self):
        data = json.loads(market_tickers.get_value().to_json(orient="records"))
        data_map = dict([(cin['instId'], cin['last']) for cin in data])
        self.total = self.usdt
        for coin in self.coin_map.values():
            coin.price = float(data_map[coin.coin_id])
            self.total += coin.money

    def to_json(self):
        return {
            "total": self.total,
            "res": round(self.usdt, 2),
            "coins": [coin.to_json() for coin in self.coin_map.values()]
        }

    @api_route("/update")
    def update_value(self, suffix=""):
        self.load_account()
        self.update_price()
        for coin in self.coin_map.values():
            coin.watch()

        return self.to_json()



import logging
import os

from notetool.secret import read_secret
from sqlalchemy import create_engine
from sqlalchemy.orm import declarative_base, sessionmaker


class BaseTable(object):
    def json(self):
        res = {k: v for k, v in self.__dict__.items() if not k.startswith("_")}
        return res


uri = read_secret(cate1='notecoin', cate2='dataset', cate3='db_path')
uri = uri or f'sqlite:///{os.path.abspath(os.path.dirname(__file__))}/data/notecoin.db'
#engine = create_engine(uri, echo=True)
engine = create_engine(uri)
Base = declarative_base()
session = sessionmaker(bind=engine)()

logging.info(f'uri:{uri}')


def create_all():
    Base.metadata.create_all(engine)
    session.commit()



import logging
import time

from notecoin.database.base import session
from notecoin.okex.database.client import OkexClientAccountBalance
from notecoin.okex.websocket.channel import PublicChannel
from notecoin.okex.websocket.connect import PublicConnect
from notecoin.okex.websocket.handle import PublicTickers
from notetool.secret import read_secret

logger = logging.getLogger()
logger.setLevel(logging.DEBUG)


def init():
    uri = "mysql+pymysql://notecoin:notecoin@127.0.0.1:3306/notecoin?charset=utf8"
    # uri = f'sqlite:///{os.path.abspath(os.path.dirname(__file__))}/notecoin.db'
    read_secret(cate1='notecoin', cate2='dataset', cate3='db_path', value=uri)


channel_update_time = time.time()


def channel_listen():
    details = session.query(OkexClientAccountBalance).filter(OkexClientAccountBalance.eqUsd > 10).all()
    channels = []
    global channel_update_time
    if time.time() < channel_update_time + 10:
        return channels
    for _detail in details:
        detail = _detail.json()
        ccy = detail['ccy']
        if ccy == 'USDT':
            continue
        channels.append(PublicChannel.public_tickers().to_json())
    channel_update_time = time.time()
    return channels


connect = PublicConnect([PublicChannel.public_tickers().to_json()], channel_listen=channel_listen)
connect.add_handle(PublicTickers())
connect.run()




class BaseModel:
    def __init__(self, name='base', *args, **kwargs):
        self.name = name

    def train(self, df, *args, **kwargs):
        pass

    def predict(self, df, *args, **kwargs):
        pass




from notetool.secret import read_secret

from .account import AccountClient
from .asset import AssetClient
from .market import MarketClient
from .public import PublicClient
from .trade import TradeClient

api_key = read_secret(cate1='coin', cate2='okex', cate3='api_key')
secret_key = read_secret(cate1='coin', cate2='okex', cate3='secret_key')
passphrase = read_secret(cate1='coin', cate2='okex', cate3='passphrase')

account_api = AccountClient(api_key, secret_key, passphrase)
funding_api = AssetClient(api_key, secret_key, passphrase)
market_api = MarketClient(api_key, secret_key, passphrase)
public_api = PublicClient(api_key, secret_key, passphrase)
trade_api = TradeClient(api_key, secret_key, passphrase)



import logging

from fastapi import APIRouter
from notebuild.tool.fastapi import add_api_routes, api_route
from notecoin.database.base import create_all, session
from notecoin.database.connect.base import BaseConnect
from notecoin.okex.database.client import OkexClientAccountBalance
from notecoin.okex.database.websocket import OkexWebsocketChannels
from notecoin.okex.websocket.channel import PublicChannel
from notecoin.okex.websocket.connect import PublicConnect


class WebsocketService(APIRouter):
    def __init__(self, connect: BaseConnect = None, prefix='/websocket', *args, **kwargs):
        self.connect = connect or PublicConnect(channels=[PublicChannel.public_tickers().to_json()])
        super(WebsocketService, self).__init__(prefix=prefix, *args, **kwargs)
        add_api_routes(self)
        self.connect.run()

    def update_channels_db(self):
        details = session.query(OkexClientAccountBalance).filter(OkexClientAccountBalance.eqUsd > 10).all()

        for _detail in details:
            detail = _detail.json()
            if detail['ccy'] == 'USDT':
                continue
            instId = f"{detail['ccy']}-USDT"
            param = {
                "channel_json": str(PublicChannel.public_tickers(instId).to_json()),
                "channel": "tickers",
                "instId": instId
            }
            session.merge(OkexWebsocketChannels(**param))
        create_all()
        session.commit()
        return {"success": len(details)}

    @api_route('/update/channel', description="get value")
    def update_channels(self):
        try:
            self.update_channels_db()

            new_channels = session.query(OkexWebsocketChannels).all()
            new_channels = [eval(channel.channel_str) for channel in new_channels]

            old_channels_str = [str(channel) for channel in self.connect.channels]
            new_channels_str = [str(channel) for channel in new_channels]
            if new_channels_str is not None and len(new_channels_str) > 0:
                if len(list(set(old_channels_str) - set(new_channels_str))) == len(
                        list(set(new_channels_str) - set(old_channels_str))) == 0:
                    logging.info(f"old:{old_channels_str},new:{new_channels_str}")
                    self.connect.channels = new_channels
                    self.connect.subscribe_restart()
            return {"update": "success"}
        except Exception as e:
            return {"update": "fails", "error": str(e)}
